;
;  Receive/transmit files over serial port
;  05/05/03
;
model4	equ	1
;model16	equ	1
;
warmst	equ	0
bdos	equ	5
fcb	equ	5ch
dma	equ	80h
tpa	equ	100h
;
cr	equ	13
lf	equ	10
backsp	equ	8
ack	equ	6
nak	equ	15h
ctrlc	equ	3
esc	equ	1bh
;
; Transfer commands
;
filecmd	equ	'F'
rdycmd	equ	'R'
closcmd	equ	'C'
blkcmd	equ	'B'
dircmd	equ	'D'
getcmd	equ	'G'
;
; User commands
;
abtcmd	equ	esc	; abort program
loccmd	equ	'L'	; get local directory
rmtcmd	equ	'R'	; get remote directory
xfcmd	equ	'T'	; transmit file
datcmd	equ	'D'	; show program date
;
	org	tpa
;
	jp	start
;
	ifdef	model16
;
; Model 16 serial port info
;
ssport	db	0f6h	; serial status port
srmask	db	01h	; data ready mask
sxmask	db	04h	; xmit ready mask
sdport	db	0f4h	; serial data port
;
	endif
;
	ifdef	model4
;
; Model 4 serial port info
;
ssport	db	0eah	; serial status port
srmask	db	80h	; data ready mask
sxmask	db	40h	; xmit ready mask
sdport	db	0ebh	; serial data port
;
	endif
;
date:	db	cr,lf,'05/05/03',cr,lf,0
;
ctable:	db	rmtcmd
	dw	rmtdir	; get remote directory
	db	loccmd
	dw	locdir	; get local directory
	db	xfcmd
	dw	xmtfile	; transmit file
	db	getcmd
	dw	getrmt	; get remote file
	db	'?'
	dw	help
	db	datcmd
	dw	showdate
	db	0
	dw	0
;
help:	ld	hl,helpmsg
	call	print
	ret
;
showdate:
	ld	hl,date
	call	print
	ret
;
start:	ld	sp,stack
	ld	hl,wmsg
	call	print
start0:	ld	b,16
	ld	hl,fname
start1:	ld	(hl),0
	inc	hl
	djnz	start1
	ld	b,36
	ld	hl,fcb
start2:	ld	(hl),0
	inc	hl
	djnz	start2
	call	getfname	; get filename from remote
	cp	a,esc		; abort if user hit ESC
	jp	z,exit
	or	a
	jr	nz,exec		; execute user command
	call	openfile
	or	a
	jp	z,exit
	ld	hl,rfmsg
	call	print
	ld	hl,fname
	call	print
	call	crlf
	ld	hl,0
	ld	(blknum),hl
	ld	a,ack
	call	sndchr
	call	readfile
	call	crlf
	ld	c,16
	ld	de,fcb
	call	bdos		; close file
	jr	start
;
exec:	call	toupper
	ld	c,a
	ld	hl,ctable
exec1:	xor	a
	or	(hl)
	jr	z,start0	; command not found
	inc	hl
	cp	c
	jr	z,exec2		; found command
	inc	hl
	inc	hl
	jr	exec1		; check next entry
;
exec2:	ld	a,(hl)
	inc	hl
	ld	h,(hl)
	ld	l,a
	ld	bc,start	; set return address
	push	bc		; on stack
	jp	(hl)		; execute command
;
; Get file name from remote
;
getfname:
	call	rcvstat
	jr	z,getfn1
	call	getser
	cp	filecmd
	jr	z,fcmd
	cp	getcmd
	jr	z,gcmd
	cp	dircmd
	jr	z,dodir
	cp	rdycmd
	jr	nz,getfn1
	ld	a,ack
	call	sndchr
getfn1:	call	kbscan
	or	a
	ret	nz
	jr	getfname
;
fcmd:	ld	hl,fname
fcmd1:	call	rcvstat
	jr	nz,fcmd2
	call	kbscan
	or	a
	jr	z,fcmd1
	ret
;
fcmd2:	call	getser
	or	a
	jr	z,fcmdx
	call	toupper
	ld	(hl),a
	inc	hl
	or	a
	jr	nz,fcmd1
fcmdx:	ld	a,ack
	call	sndchr
	xor	a
	ret
;
gcmd:	call	fcmd		; get filename template
	ld	hl,gcmdmsg
	call	print
	ld	hl,fname
	call	print
	call	crlf
	call	xmtf0
	ld	hl,wmsg
	call	print
	jp	getfname
;
; Send directory list to remote.
;
dodir:	call	fcmd		; get dir template
	ld	hl,dirmsg
	call	print
	ld	hl,fname
	call	print
	call	crlf
;
	call	makefilelist
	ld	a,(filecnt)
	or	a
	jr	nz,dodir1	; go if files found
	ld	a,closcmd
	call	sndchr
	ld	hl,nofmsg
	call	print
	ld	a,1
	ret
;
dodir1:	ld	b,a		; save file count
	ld	de,0
dodir2:	push	de
	ex	de,hl
	add	hl,hl		; x 2
	add	hl,hl		; x 4
	add	hl,hl		; x 8
	add	hl,hl		; x 16
	ld	de,filelist
	add	hl,de
	pop	de
	inc	de
	call	convertfname
	ld	hl,fname
	ld	a,filecmd	; send command code
	call	sndchr
dodir3:	ld	a,(hl)		; send file name
	inc	hl
	or	a
	jr	z,dodir4
	call	sndchr
	jr	dodir3
;
dodir4:	call	sndchr
	djnz	dodir2
	ld	a,closcmd
	call	sndchr
	ld	a,1
	ret
;
; Expand wildcards in filename
;
expwc:	ld	hl,fname	; transfer filename
	ld	de,tempf
	ld	bc,16
	ldir
	ld	hl,tempf
	ld	de,fname
	ld	b,8		; file name char count
expwc1:	ld	a,(hl)
	inc	hl
	or	a
	jr	z,expwcx
	cp	'*'
	jr	z,expwc2
	cp	'.'
	jr	z,expwce	; early termination of file name
	ld	(de),a
	inc	de
	djnz	expwc1
	ld	a,(hl)
	inc	hl
	cp	'.'
	jr	z,expwce	; fn done, do ext
	jr	expwcx		; if any
;
expwc2:	ld	a,'?'
	ld	(de),a
	inc	de
	djnz	expwc2
	ld	a,(hl)		; extension?
	inc	hl
	cp	'.'
	jr	nz,expwcx	; go if not
;
expwce:	ld	b,3		; now do extension
	ld	(de),a		; put '.' in fname
	inc	de
expwc3:	ld	a,(hl)
	inc	hl
	or	a
	jr	z,expwcx	; done if end of ext
	cp	'*'
	jr	z,expwc4
	ld	(de),a
	inc	de
	djnz	expwc3
	jr	expwcx
;
expwc4:	ld	a,'?'
	ld	(de),a
	inc	de
	djnz	expwc4
expwcx:	xor	a
	ld	(de),a		; terminate expanded filename
	ret
;
; Exit program
;
exit:	ld	hl,exitmsg
	call	print
	jp	warmst
;
; Convert character in A to uppper case
;
toupper:
	cp	'a'
	ret	c
	cp	'z'+1
	ret	nc
	sub	20h
	ret
;
; Open file.
; Return 0 if failure.
;
openfile:
	ld	hl,fname
opfil1:	ld	a,(hl)
	inc	hl
	or	a
	jr	z,opfil2
	cp	'*'		; check for wildcards
	jr	z,opfilf	; abort if any found
	cp	'?'
	jr	nz,opfil1
opfilf:	xor	a
	ret

opfil2:	call	setfcb
	call	fopen
	ret
;
; Setup FCB from filename
;
setfcb:	ld	hl,fcb
	ld	de,fname
	ld	(hl),0
	inc	hl
	ld	b,8
stfcb1:	ld	a,(de)
	cp	'.'
	jr	z,stfcb2
	or	a
	jr	z,stfcb2
	ld	(hl),a
	inc	hl
	inc	de
	djnz	stfcb1
stfcb2:	ld	a,b
	or	a
	jr	z,stfcb4
stfcb3:	ld	(hl),' '
	inc	hl
	djnz	stfcb3
stfcb4:	ld	hl,fcb+9
	ld	b,3
	ld	a,(de)
	cp	'.'
	jr	nz,stfcb5
	inc	de
stfcb5:	ld	a,(de)
	or	a
	jr	z,stfcb6
	ld	(hl),a
	inc	hl
	inc	de
	djnz	stfcb5
stfcb6:	ld	a,b
	or	a
	jr	z,stfcb8
stfcb7:	ld	(hl),' '
	inc	hl
	djnz	stfcb7
stfcb8:	ret
;
; Create output file. If a file with the same
; name exists, delete it. Return 0 if failed.
;
fopen:	ld	c,17
	ld	de,fcb		; see if file exists by
	call	bdos		; doing a search for first
	inc	a
	jr	z,fopen1	; ok, file does not exist
	ld	c,19		; else delete it
	ld	de,fcb
	call	bdos
fopen1:	ld	c,22
	ld	de,fcb
	call	bdos		; create output file
	inc	a
	ret	nz
	ld	hl,nfmsg
	call	print
	xor	a
	ret
;
; Receive file sent from remote.
; We get here as a result of receiving
; an FCMD command after the file name
; has been received.
;
readfile:
	call	kbscan
	cp	esc
	ret	z
	call	rcvstat
	jr	z,readfile
	call	getser
	cp	closcmd
	ret	z
	cp	blkcmd
	jr	nz,readfile
;
; should be block number here...
;
	ld	a,rdycmd
	call	sndchr
	call	getchr
	ld	b,a
	call	getchr
	ld	c,a
	ld	hl,(blknum)
	or	a
	sbc	hl,bc
	jr	z,rdfil1
;
	push	bc
	ld	hl,bnemsg
	call	print
	ld	bc,(blknum)
	call	prtdec
	ld	hl,bnemsg1
	call	print
	pop	bc
	call	prtdec
;
	ld	a,nak
	call	sndchr
	ld	a,1
	ret
;
bnemsg:	db	cr,lf,'Block number error!'
	db	cr,lf,'Current block: ',0
bnemsg1:db	cr,lf,'Received: ',0
;
rdfil1:	call	readblock
	or	a
	ret	nz
	jp	readfile
;
readblock:
	ld	hl,bnumsg
	call	print
	ld	hl,blknum
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	prtdec
	inc	bc
	ld	(blknum),bc
	ld	a,rdycmd
	call	sndchr
	ld	b,128
	ld	hl,dma
rblk1:	call	rcvstat
	jr	nz,rblk2
	call	kbscan
	cp	esc
	jr	nz,rblk1
	ret
;
rblk2:	call	getser
	ld	(hl),a
	inc	hl
	djnz	rblk1
	ld	c,15h
	ld	de,fcb
	call	bdos		; write block to file
;
;	ld	a,'.'
;	call	chrout
;
	ld	a,ack
	call	sndchr
	xor	a
	ret
;
crlf:	ld	a,cr
	call	chrout
	ld	a,lf
;
chrout:	push	hl
	push	de
	push	bc
	push	af
	ld	e,a
	ld	c,6
	call	bdos
	pop	af
bdosrt:	pop	bc
	pop	de
	pop	hl
	ret
;
chrin:	call	kbscan
	jr	z,chrin
	ret
;
kbscan:	push	hl
	push	de
	push	bc
	ld	e,0ffh
	ld	c,6
	call	bdos
	or	a
	jr	bdosrt
;
print:	ld	a,(hl)
	inc	hl
	or	a
	ret	z
	call	chrout
	jr	print
;
sndchr:	push	de
	push	bc
	ld	e,a
	ld	c,4
	call	sys
	pop	bc
	pop	de
	ret
;
sys:	push	hl
	push	de
	push	bc
	call	bdos
	pop	bc
	pop	de
	pop	hl
	ret
;
; Get character from serial port.
; This routine does not return
; until a character is available.
;
getchr:
;
;	ld	c,3
;	call	sys
;	ret
;
	call	rcvstat
	or	a
	jr	z,getchr
	call	getser
	ret
;
; Check for serial receive char.
; Returns true if char avail.
;
rcvstat:
	push	hl
	push	bc
	ld	hl,ssport
	ld	c,(hl)
	in	a,(c)
	inc	hl
	and	(hl)
	pop	bc
	pop	hl
	ret
;
; Get serial receive character.
;
getser:	push	hl
	push	bc
	ld	hl,sdport
	ld	c,(hl)
	in	a,(c)
	pop	bc
	pop	hl
	ret
;
; Check serial transmit.
; Return true if ready to send.
;
xmtstat:
	push	hl
	push	bc
	ld	hl,ssport
	ld	c,(hl)
	inc	hl
	inc	hl
	in	a,(c)
	and	(hl)
	pop	bc
	pop	hl
	ret
;
; Send char in A to serial output.
;
putser:	push	hl
	push	bc
	ld	hl,sdport
	ld	c,(hl)
	out	(c),a
	pop	bc
	pop	hl
	ret
;
; Get file spec from user and put in fname.
;
getfspec:
	ld	hl,efsmsg
	call	print
	ld	hl,fname
	ld	b,0		; character count
gfs1:	call	chrin
	cp	esc
	jr	z,gfsabt
	cp	cr
	jr	z,gfsx
	cp	backsp
	jr	z,gfsbs
	cp	'.'
	jr	z,gfs2
	cp	'*'
	jr	z,gfs2
	cp	'?'
	jr	z,gfs2
	call	isalnum
	jr	c,gfs1		; ignore if not alphanumeric
gfs2:	call	chrout		; else display it
	call	toupper
	ld	(hl),a		; and save it
	inc	hl
	inc	b		; increment byte count
	jr	gfs1
;
gfsbs:	ld	a,b
	or	a
	jr	z,gfs1
	ld	a,backsp
	call	chrout
	ld	a,' '
	call	chrout
	ld	a,backsp
	call	chrout
	dec	hl
	dec	b
	jr	gfs1
;
gfsx:	ld	(hl),0
	ld	a,b		; return byte count
	ret
;
gfsabt:	xor	a
	ret
;
; Check if character in A is alphanumeric.
; Return C if not, else return NC.
;
isalnum:
	call	isnum
	ret	nc
	call	isalpha
	ret
;
; Check if character in A is alpha.
; Return C if not, else return NC.
;
isalpha:
	cp	'A'
	jr	c,isalpn
	cp	'['
	jr	c,isalpy
	cp	'a'
	jr	c,isalpn
	cp	'{'
	jr	c,isalpy
isalpn:	ret
;
isalpy:	ccf
	ret
;
; Check if character in A is numeric.
; Return C if not, else return NC.
;
isnum:	cp	'0'
	jr	c,isnumn
	cp	':'
	jr	nc,isnumn
	ccf
	ret
;
isnumn:	scf
	ret
;
; Check if character in A is printable.
; Return C if not, else return NC.
;
isprint:
	cp	'!'
	jr	c,isprtn
	cp	'~'+1
	jr	nc,isprtn
	scf
	ccf
	ret
;
isprtn:	scf
	ret
;
; Get remote file
;
getrmt:	ld	hl,getmsg
	call	print
	call	getfspec
	ld	a,getcmd
	call	sndchr
	ld	hl,fname
getrm1:	ld	a,(hl)
	or	a
	jr	z,getrm2
	inc	hl
	call	sndchr
	jr	getrm1
;
getrm2:	xor	a
	call	sndchr
	ret
;
; Remote directory list
;
rmtdir:	ld	hl,rdmsg
	call	print
	call	getfspec
	call	crlf
	ld	a,dircmd
	call	sndchr
	ld	hl,fname
rmtdr1:	ld	a,(hl)
	or	a
	jr	z,rmtdr2
	inc	hl
	call	sndchr
	jr	rmtdr1
;
rmtdr2:	xor	a
	call	sndchr
	ld	c,0		; init filename count
rmtdr3:	call	rcvstat
	jr	z,rmtdr4
	call	getser
	cp	filecmd
	jr	z,rmtdr5
	cp	closcmd
	ret	z
	jr	rmtdr3
;
rmtdr4:	call	kbscan
	or	a
	ret	nz
	jr	rmtdr3
;
rmtdr5:	ld	hl,fname	; got file command
	ld	b,0		; init char count
rmtdr6:	call	rcvstat
	jr	z,rmtdr7
	call	getser
	ld	(hl),a
	inc	hl
	or	a		; end of file name
	jr	z,rmtdr8
	inc	b
	jr	rmtdr6
;
rmtdr7:	call	kbscan
	or	a
	ret	nz
	jr	rmtdr6
;
rmtdr8:	ld	hl,fname	; got file name
	inc	c
	ld	d,b		; save char count
rmtdr9:	ld	a,(hl)
	inc	hl
	call	chrout
	djnz	rmtdr9
	ld	a,c
	cp	5
	jr	c,rmtdra	; go if haven't got 5 filenames yet
	call	crlf
	ld	a,ack
	call	sndchr
	ld	c,0
	jr	rmtdr3
;
rmtdra:	ld	a,d		; recover char count
	cp	15
	jr	nc,rmtdrb
	inc	d
	ld	a,' '
	call	chrout
	jr	rmtdra
;
rmtdrb:	ld	a,ack
	call	sndchr
	jr	rmtdr3
;
; Local directory list
;
locdir:	ld	hl,ldmsg
	call	print
	call	getfspec
	or	a
	ret	z
	call	makefilelist
	ld	a,(filecnt)
	or	a
	jr	nz,ldir1
	ld	hl,nofmsg
	call	print
	ret
;
ldir1:	call	crlf
	ld	a,(filecnt)
	ld	b,a		; save file count
	ld	de,0
ldir2:	push	de
	ld	h,0
	ld	l,e
	add	hl,hl		; x 2
	add	hl,hl		; x 4
	add	hl,hl		; x 8
	add	hl,hl		; x 16
	ld	de,filelist
	add	hl,de
	pop	de
	inc	e
	call	convertfname
	ld	c,0
	ld	hl,fname
ldir3:	ld	a,(hl)		; print file name
	inc	hl
	or	a
	jr	z,ldirn
	call	chrout
	inc	c
	jr	ldir3
;
ldirn:	inc	d
	ld	a,d
	cp	5
	jr	c,ldirn1
	ld	d,0
	call	crlf
	jr	ldirn2
;
ldirn1:	ld	a,c
	cp	16
	jr	nc,ldirn2
	ld	a,' '
	call	chrout
	inc	c
	jr	ldirn1
;
ldirn2:	djnz	ldir2
	call	crlf
	ret
;
; Send file(s) to remote.
; Opening a file DESTROYS info needed by BDOS
; search next function (stupid bdos!), so we
; need to get all the file names before
; sending the first one.
;
xmtfile:
	call	crlf
	call	getfspec
	call	crlf
xmtf0:	call	makefilelist
	xor	a		; start sending files
	ld	(filenum),a
xmtf1:	ld	a,(filenum)
	ld	hl,filecnt
	cp	(hl)
	jr	z,xmtf3
	inc	a
	ld	(filenum),a
	dec	a
	ld	l,a
	ld	h,0
	add	hl,hl		; x 2
	add	hl,hl		; x 4
	add	hl,hl		; x 8
	add	hl,hl		; x 16
	ld	de,filelist
	add	hl,de		; offset into filelist
	ld	de,fcb		; transfer file name to fcb
	ld	bc,12
	ldir
	ex	de,hl
	ld	b,24
xmtf2:	ld	(hl),0
	inc	hl
	djnz	xmtf2
	call	sendfile
	or	a
	jr	z,xmtf1		; do next file
xmtf3:	ld	hl,donemsg
	call	print
	ret
;
; Send file to remote.
; Return ESC if user aborts, 0 if ok,
; or nonzero if error.
;
sendfile:
	ld	c,15
	ld	de,fcb
	call	bdos		; open file
	inc	a
	jr	nz,sfile1
	ld	a,1
	ret
;
sfile1:	ld	hl,sfmsg
	call	print
	ld	hl,fcb+1
	ld	a,filecmd	; send command code
	call	sndchr
	ld	b,8
sfile2:	ld	a,(hl)		; send file name
	inc	hl
	cp	' '
	jr	z,sfile3
	call	chrout
	call	sndchr
	djnz	sfile2
sfile3:	ld	a,b
	or	a
	jr	z,sfile5
	dec	hl
sfile4:	inc	hl
	djnz	sfile4
sfile5:	ld	a,(hl)
	cp	' '
	jr	z,sfile7	; go if no file extension
	ld	a,'.'
	call	chrout
	call	sndchr		; else send file ext
	ld	b,3
sfile6:	ld	a,(hl)
	inc	hl
	cp	' '
	jr	z,sfile7
	call	chrout
	call	sndchr
	djnz	sfile6
sfile7:	call	crlf
	xor	a
	call	sndchr
	call	getack
	jr	nc,sfile8	; go if ack or abort
	ld	hl,0
	ld	(blknum),hl
sfile8:	call	rdblk
	or	a
	jr	nz,sfile9
;
;	ld	a,'.'
;	call	chrout
;
	call	sndblk
	or	a
	jr	z,sfile8
	ld	c,16
	ld	de,fcb
	call	bdos		; close file
	ld	hl,xfmsg
	call	print
	ld	a,1
	ret
;
sfile9:	ld	c,16
	ld	de,fcb
	call	bdos		; close file
	ld	a,closcmd
	call	sndchr
	call	getack
	ret
;
rdblk:	ld	c,20
	ld	de,fcb
	call	bdos
	ret
;
; Send next block to remote.
; Return 0 if ok, 1 if error,
; or esc if user abort.
;
sndblk:	ld	bc,(blknum)
	ld	a,blkcmd
	call	sndchr
	ld	a,b
	call	sndchr
	ld	a,c
	call	sndchr
	ld	hl,bnumsg
	call	print
	call	prtdec
	inc	bc
	ld	(blknum),bc
	call	getack
	jr	nc,sblke	; go if not ack
	ld	hl,dma
	ld	b,128
sblk1:	ld	a,(hl)
	inc	hl
	call	sndchr
	djnz	sblk1
	call	getack
	jr	c,sblk2		; go if no error
	cp	esc
	ret	z
sblke:	ld	a,1		; return error
	ret
;
sblk2:	xor	a		; return no error
	ret
;
; Wait for ack from remote.
; return C if ack received, else NC.
; A contains ack or keyboard character.
;
getack:	call	rcvstat
	jr	z,getak1
	call	getser
	cp	ack
	jr	z,getaky
	xor	a
	ret
;
getaky:	xor	a
	scf
	ret
;
getak1:	call	kbscan
	or	a
	ret	nz
	jr	getack
;
; Make file list. Wildcarded file spec
; must be in FNAME. On exit, FILECNT will
; contain the number of entries in
; FILELIST. Each entry is 16 bytes.
;
makefilelist:
	xor	a
	ld	(filecnt),a
	call	expwc		; expand wildcards
	call	setfcb
	ld	de,fcb
	ld	c,17
	call	bdos		; search for first
	inc	a
	jr	nz,mkfl1
	ld	hl,nofmsg
	call	print
	ld	a,1
	ret
;
mkfl1:	ld	de,filelist
mkfl2:	dec	a
	rlca
	rlca
	rlca
	rlca
	rlca
	ld	c,a
	ld	b,0
	ld	hl,dma
	add	hl,bc		; hl -> directory entry
	ld	bc,12		; transfer filename to filelist
	ldir
	ex	de,hl
	ld	b,4
mkfl3:	ld	(hl),0		; clear remainder of file entry
	inc	hl
	djnz	mkfl3
	ex	de,hl
	ld	hl,filecnt
	inc	(hl)
	push	de		; save filelist pointer
	ld	c,18
	call	bdos		; search for next
	inc	a
	pop	de		; recover filelist pointer
	jr	nz,mkfl2	; go if found
	ret
;
; Convert file name from FCB format to fname.
; On entry, hl -> fcb format file name.
; On exit, fname contains DOS type filename.
;
convertfname:
	push	hl
	push	de
	push	bc
	ld	b,8
	ld	de,fname
	inc	hl		; skip over drive number
cfn1:	ld	a,(hl)		; send file name
	inc	hl
	cp	' '
	jr	z,cfn2
	ld	(de),a
	inc	de
	djnz	cfn1
cfn2:	ld	a,b
	or	a
	jr	z,cfn4
	dec	hl
cfn3:	inc	hl
	djnz	cfn3
cfn4:	ld	a,(hl)
	cp	' '
	jr	z,cfn6		; go if no file extension
	ld	a,'.'
	ld	(de),a
	inc	de		; else set file ext
	ld	b,3
cfn5:	ld	a,(hl)
	inc	hl
	cp	' '
	jr	z,cfn6
	ld	(de),a
	inc	de
	djnz	cfn5
cfn6:	xor	a
	ld	(de),a
	pop	bc
	pop	de
	pop	hl
	ret
;
; Output unsigned number in BC in decimal
; leading zeros are suppressed
;
prtdec:	push	hl
	push	de
	push	bc
	ld	h,b
	ld	l,c
	ld	d,0
	ld	bc,10000
	call	pdecs
	call	pdecc
	ld	bc,1000
	call	pdecs
	call	pdecc
	ld	bc,100
	call	pdecs
	call	pdecc
	ld	bc,10
	call	pdecs
	call	pdecc
	ld	a,l
	add	'0'
	call	chrout
	pop	bc
	pop	de
	pop	hl
	ret
;
pdecc:	or	a
	jr	nz,pdecc1
	or	d
	ret	z
	xor	a
	jr	pdecc2
;
pdecc1:	ld	d,1
pdecc2:	add	'0'
	call	chrout
	ret
;
pdecs:	ld	e,0ffh
pdecs1:	inc	e
	or	a		; clear carry
	sbc	hl,bc
	jr	nc,pdecs1
	add	hl,bc
	ld	a,e
	ret
;
blknum:	defs	2
;
wmsg:	db	cr,lf,'Waiting for file...',,0
rfmsg:	db	cr,lf,'Receiving file ',0
nfmsg:	db	cr,lf,'Cant''t create file',cr,lf,0
sfmsg:	db	cr,lf,'Sending file ',0
bnumsg:	db	cr,'Block ',0
xfmsg:	db	cr,lf,'Transfer failed',cr,lf,0
;
ldmsg:	db	cr,lf,'Local directory:',cr,lf,0
efsmsg:	db	'Enter file spec: ',0
rdmsg:	db	cr,lf,'Remote directory:',cr,lf,0
getmsg:	db	cr,lf,'Get remote file:',cr,lf,0
gcmdmsg:db	cr,lf,'File request: ',0
;
dirmsg:	db	cr,lf,'Directory request for: ',0
nofmsg:	db	cr,lf,'No files match',cr,lf,0
;
donemsg:db	cr,lf,'Done',cr,lf,0
;
helpmsg:
	db	cr,lf,'Commands:',cr,lf
	db	'L - local directory',cr,lf
	db	'R - remote directory',cr,lf
	db	'G - get file from remote',cr,lf
	db	'T - transmit file to remote',cr,lf
	db	'<ESC> - end program',cr,lf
	db	cr,lf,0
;
exitmsg:db	cr,lf,'Exiting...',cr,lf,0
;
tempf:	defs	16
fname:	defs	16
filecnt:defs	1
filenum:defs	1
;
	defs	100
stack	equ	$
;
filelist	equ	$
;
	end
;

